@@ -2,11 +2,12 @@ from pydfs_lineup_optimizer.sites.draftkings.classic.importer import DraftKingsC
 from pydfs_lineup_optimizer.sites.draftkings.classic.settings import *
 from pydfs_lineup_optimizer.sites.draftkings.captain_mode.importer import DraftKingsCaptainModeCSVImporter
 from pydfs_lineup_optimizer.sites.draftkings.captain_mode.settings import *
+from pydfs_lineup_optimizer.sites.draftkings.tiers.settings import *
 
 __all__ = [
     'DraftKingsCSVImporter', 'DraftKingsBasketballSettings', 'DraftKingsFootballSettings',
     'DraftKingsHockeySettings', 'DraftKingsBaseballSettings', 'DraftKingsGolfSettings',
     'DraftKingsSoccerSettings', 'DraftKingsCanadianFootballSettings', 'DraftKingsLOLSettings',
     'DraftKingsCaptainModeCSVImporter', 'DraftKingsCaptainModeFootballSettings',
-    'DraftKingsCaptainModeBasketballSettings',
+    'DraftKingsCaptainModeBasketballSettings', 'DraftKingsTiersBasketballSettings',
 ]
