@@ -4,6 +4,7 @@ from pydfs_lineup_optimizer.constants import Sport, Site
 from pydfs_lineup_optimizer.sites.sites_registry import SitesRegistry
 from pydfs_lineup_optimizer.lineup_printer import IndividualSportLineupPrinter
 from pydfs_lineup_optimizer.sites.fantasy_draft.importer import FantasyDraftCSVImporter
+from pydfs_lineup_optimizer.lineup_exporter import FantasyDraftCSVLineupExporter
 
 
 class FantasyDraftSettings(BaseSettings):
@@ -11,6 +12,7 @@ class FantasyDraftSettings(BaseSettings):
     budget = 100000
     max_from_one_team = 6  # type: Optional[int]
     csv_importer = FantasyDraftCSVImporter
+    csv_exporter = FantasyDraftCSVLineupExporter
 
 
 @SitesRegistry.register_settings
